@@ -1,28 +1,44 @@
+/*-------------------------------------------------------------------------
+ *
+ * AbstractJdbc1ResultSet.java
+ *     This class defines methods of the jdbc1 specification.  This class is
+ *     extended by org.postgresql.jdbc2.AbstractJdbc2ResultSet which adds the
+ *     jdbc2 methods.  The real ResultSet class (for jdbc1) is 
+ *     org.postgresql.jdbc1.Jdbc1ResultSet
+ *
+ * Copyright (c) 2003, PostgreSQL Global Development Group
+ *
+ * IDENTIFICATION
+ *	  $Header: /cvsroot/pgsql-server/src/interfaces/jdbc/org/postgresql/jdbc1/AbstractJdbc1ResultSet.java,v 1.22.2.1 2003/12/12 17:59:08 davec Exp $
+ *
+ *-------------------------------------------------------------------------
+ */
 package org.postgresql.jdbc1;
 
-
 import java.math.BigDecimal;
 import java.io.*;
 import java.sql.*;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.Vector;
-import org.postgresql.Field;
+import org.postgresql.Driver;
+import org.postgresql.core.BaseConnection;
+import org.postgresql.core.BaseResultSet;
+import org.postgresql.core.BaseStatement;
+import org.postgresql.core.Field;
 import org.postgresql.core.Encoding;
+import org.postgresql.core.QueryExecutor;
 import org.postgresql.largeobject.*;
 import org.postgresql.util.PGbytea;
+import org.postgresql.util.PGtokenizer;
 import org.postgresql.util.PSQLException;
+import org.postgresql.util.PSQLState;
 
-/* $Header: /cvsroot/pgsql-server/src/interfaces/jdbc/org/postgresql/jdbc1/Attic/AbstractJdbc1ResultSet.java,v 1.7.2.1 2003/01/14 09:15:35 barry Exp $
- * This class defines methods of the jdbc1 specification.  This class is
- * extended by org.postgresql.jdbc2.AbstractJdbc2ResultSet which adds the jdbc2
- * methods.  The real ResultSet class (for jdbc1) is org.postgresql.jdbc1.Jdbc1ResultSet
- */
-public abstract class AbstractJdbc1ResultSet
+public abstract class AbstractJdbc1ResultSet implements BaseResultSet
 {
 
 	protected Vector rows;			// The results
-	protected Statement statement;
+	protected BaseStatement statement;
 	protected Field fields[];		// The field descriptions
 	protected String status;		// Status of the result
 	protected boolean binaryCursor = false; // is the data binary or Strings
@@ -30,40 +46,128 @@ public abstract class AbstractJdbc1ResultSet
 	protected long insertOID;		// The oid of an inserted row
 	protected int current_row;		// Our pointer to where we are at
 	protected byte[][] this_row;		// the current row result
-	protected org.postgresql.PGConnection connection;	// the connection which we returned from
+	protected BaseConnection connection;	// the connection which we returned from
 	protected SQLWarning warnings = null;	// The warning chain
 	protected boolean wasNullFlag = false;	// the flag for wasNull()
 
 	// We can chain multiple resultSets together - this points to
 	// next resultSet in the chain.
-	protected ResultSet next = null;
+	protected BaseResultSet next = null;
 
-	protected StringBuffer sbuf = null;
+	private StringBuffer sbuf = null;
 	public byte[][] rowBuffer = null;
 
+ 	private SimpleDateFormat m_tsFormat = null;
+ 	private SimpleDateFormat m_tstzFormat = null;
+ 	private SimpleDateFormat m_dateFormat = null;
 
-	public AbstractJdbc1ResultSet(org.postgresql.PGConnection conn, Statement statement, Field[] fields, Vector tuples, String status, int updateCount, long insertOID, boolean binaryCursor)
+	private int fetchSize;      // Fetch size for next read (might be 0).
+	private int lastFetchSize;  // Fetch size of last read (might be 0).
+
+	public abstract ResultSetMetaData getMetaData() throws SQLException;
+
+	public AbstractJdbc1ResultSet(BaseStatement statement,
+				      Field[] fields,
+				      Vector tuples,
+				      String status,
+				      int updateCount,
+				      long insertOID, 
+					  boolean binaryCursor)
 	{
-		this.connection = conn;
+		this.connection = statement.getPGConnection();
 		this.statement = statement;
 		this.fields = fields;
 		this.rows = tuples;
 		this.status = status;
 		this.updateCount = updateCount;
+
+		this.insertOID = insertOID;
+		this.this_row = null;
+		this.current_row = -1;
+		this.binaryCursor = binaryCursor;
+
+		this.lastFetchSize = this.fetchSize = (statement == null ? 0 : statement.getFetchSize());
+	}
+
+    public BaseStatement getPGStatement() {
+		return statement;
+	}
+
+	public StringBuffer getStringBuffer() {
+		return sbuf;
+	}
+
+	//This is implemented in jdbc2
+	public void setStatement(BaseStatement statement) {
+	}
+
+	//method to reinitialize a result set with more data
+	public void reInit (Field[] fields, Vector tuples, String status,
+			  int updateCount, long insertOID, boolean binaryCursor)
+	{
+		this.fields = fields;
+		// on a reinit the size of this indicates how many we pulled
+		// back. If it's 0 then the res set has ended.
+		this.rows = tuples;
+		this.status = status;
+		this.updateCount = updateCount;
 		this.insertOID = insertOID;
 		this.this_row = null;
 		this.current_row = -1;
 		this.binaryCursor = binaryCursor;
 	}
 
+	//
+	// Part of the JDBC2 support, but convenient to implement here.
+	//
+  
+	public void setFetchSize(int rows) throws SQLException
+	{
+		fetchSize = rows;
+	}
+
+
+	public int getFetchSize() throws SQLException
+	{
+		return fetchSize;
+	}
 
 	public boolean next() throws SQLException
 	{
 		if (rows == null)
-			throw new PSQLException("postgresql.con.closed");
+			throw new PSQLException("postgresql.con.closed", PSQLState.CONNECTION_DOES_NOT_EXIST);
 
 		if (++current_row >= rows.size())
-			return false;
+		{
+ 			String cursorName = statement.getFetchingCursorName();
+			if (cursorName == null || lastFetchSize == 0 || rows.size() < lastFetchSize)
+				return false;  // Not doing a cursor-based fetch or the last fetch was the end of the query
+ 
+  			// Use the ref to the statement to get
+  			// the details we need to do another cursor
+  			// query - it will use reinit() to repopulate this
+  			// with the right data.
+ 
+ 			// NB: We can reach this point with fetchSize == 0 
+ 			// if the fetch size is changed halfway through reading results.
+ 			// Use "FETCH FORWARD ALL" in that case to complete the query.
+ 			String[] sql = new String[] {
+ 				fetchSize == 0 ? ("FETCH FORWARD ALL FROM " + cursorName) :
+ 				("FETCH FORWARD " + fetchSize + " FROM " + cursorName)
+ 			};
+ 
+  			QueryExecutor.execute(sql,
+ 								  new String[0],
+  								  this);
+  
+  			// Test the new rows array.
+ 			lastFetchSize = fetchSize;
+  			if (rows.size() == 0)
+  				return false;
+
+			// Otherwise reset the counter and let it go on...
+			current_row = 0;
+		}
 
 		this_row = (byte [][])rows.elementAt(current_row);
 
@@ -94,7 +198,7 @@ public abstract class AbstractJdbc1ResultSet
 			return null;
 
 		Encoding encoding = connection.getEncoding();
-		return encoding.decode(this_row[columnIndex - 1]);
+		return trimString(columnIndex, encoding.decode(this_row[columnIndex-1]));
 	}
 
 	public boolean getBoolean(int columnIndex) throws SQLException
@@ -111,11 +215,28 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				switch(fields[columnIndex-1].getSQLType())
+				{
+					case Types.NUMERIC:
+					case Types.REAL:
+					case Types.DOUBLE:
+					case Types.FLOAT:
+					case Types.DECIMAL:
+						int loc = s.indexOf(".");
+						if (loc!=-1 && Integer.parseInt(s.substring(loc+1,s.length()))==0)
+						{
+							s = s.substring(0,loc);
+						}
+						break;
+					case Types.CHAR:
+						s = s.trim();
+						break;
+				}
 				return Byte.parseByte(s);
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException("postgresql.res.badbyte", s);
+				throw new PSQLException("postgresql.res.badbyte", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0; // SQL NULL
@@ -129,11 +250,28 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				switch(fields[columnIndex-1].getSQLType())
+				{
+					case Types.NUMERIC:
+					case Types.REAL:
+					case Types.DOUBLE:
+					case Types.FLOAT:
+					case Types.DECIMAL:
+						int loc = s.indexOf(".");
+						if (loc!=-1 && Integer.parseInt(s.substring(loc+1,s.length()))==0)
+						{
+							s = s.substring(0,loc);
+						}
+						break;
+					case Types.CHAR:
+						s = s.trim();
+						break;
+				}
 				return Short.parseShort(s);
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException("postgresql.res.badshort", s);
+				throw new PSQLException("postgresql.res.badshort", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0; // SQL NULL
@@ -190,16 +328,16 @@ public abstract class AbstractJdbc1ResultSet
 				//If the data is already binary then just return it
 				return this_row[columnIndex - 1];
 			}
-			else if (((AbstractJdbc1Connection)connection).haveMinimumCompatibleVersion("7.2"))
+			else if (connection.haveMinimumCompatibleVersion("7.2"))
 			{
 				//Version 7.2 supports the bytea datatype for byte arrays
 				if (fields[columnIndex - 1].getPGType().equals("bytea"))
 				{
-					return PGbytea.toBytes(this_row[columnIndex - 1]);
+					return trimBytes(columnIndex, PGbytea.toBytes(this_row[columnIndex - 1]));
 				}
 				else
 				{
-					return this_row[columnIndex - 1];
+					return trimBytes(columnIndex, this_row[columnIndex - 1]);
 				}
 			}
 			else
@@ -212,11 +350,11 @@ public abstract class AbstractJdbc1ResultSet
 					LargeObject lob = lom.open(getInt(columnIndex));
 					byte buf[] = lob.read(lob.size());
 					lob.close();
-					return buf;
+					return trimBytes(columnIndex, buf);
 				}
 				else
 				{
-					return this_row[columnIndex - 1];
+					return trimBytes(columnIndex, this_row[columnIndex - 1]);
 				}
 			}
 		}
@@ -230,12 +368,12 @@ public abstract class AbstractJdbc1ResultSet
 
 	public Time getTime(int columnIndex) throws SQLException
 	{
-		return toTime( getString(columnIndex), (java.sql.ResultSet)this, fields[columnIndex - 1].getPGType() );
+		return toTime( getString(columnIndex), this, fields[columnIndex - 1].getPGType() );
 	}
 
 	public Timestamp getTimestamp(int columnIndex) throws SQLException
 	{
-		return toTimestamp( getString(columnIndex), (java.sql.ResultSet)this, fields[columnIndex - 1].getPGType() );
+		return toTimestamp( getString(columnIndex), this, fields[columnIndex - 1].getPGType() );
 	}
 
 	public InputStream getAsciiStream(int columnIndex) throws SQLException
@@ -245,7 +383,7 @@ public abstract class AbstractJdbc1ResultSet
 		if (wasNullFlag)
 			return null;
 
-		if (((AbstractJdbc1Connection)connection).haveMinimumCompatibleVersion("7.2"))
+		if (connection.haveMinimumCompatibleVersion("7.2"))
 		{
 			//Version 7.2 supports AsciiStream for all the PG text types
 			//As the spec/javadoc for this method indicate this is to be used for
@@ -259,7 +397,7 @@ public abstract class AbstractJdbc1ResultSet
 			}
 			catch (UnsupportedEncodingException l_uee)
 			{
-				throw new PSQLException("postgresql.unusual", l_uee);
+				throw new PSQLException("postgresql.unusual", PSQLState.UNEXPECTED_ERROR, l_uee);
 			}
 		}
 		else
@@ -276,7 +414,7 @@ public abstract class AbstractJdbc1ResultSet
 		if (wasNullFlag)
 			return null;
 
-		if (((AbstractJdbc1Connection)connection).haveMinimumCompatibleVersion("7.2"))
+		if (connection.haveMinimumCompatibleVersion("7.2"))
 		{
 			//Version 7.2 supports AsciiStream for all the PG text types
 			//As the spec/javadoc for this method indicate this is to be used for
@@ -290,7 +428,7 @@ public abstract class AbstractJdbc1ResultSet
 			}
 			catch (UnsupportedEncodingException l_uee)
 			{
-				throw new PSQLException("postgresql.unusual", l_uee);
+				throw new PSQLException("postgresql.unusual", PSQLState.UNEXPECTED_ERROR, l_uee);
 			}
 		}
 		else
@@ -307,7 +445,7 @@ public abstract class AbstractJdbc1ResultSet
 		if (wasNullFlag)
 			return null;
 
-		if (((AbstractJdbc1Connection)connection).haveMinimumCompatibleVersion("7.2"))
+		if (connection.haveMinimumCompatibleVersion("7.2"))
 		{
 			//Version 7.2 supports BinaryStream for all PG bytea type
 			//As the spec/javadoc for this method indicate this is to be used for
@@ -433,7 +571,7 @@ public abstract class AbstractJdbc1ResultSet
 
 	public String getCursorName() throws SQLException
 	{
-		return ((AbstractJdbc1Connection)connection).getCursorName();
+		return (connection.getCursorName());
 	}
 
 	/*
@@ -456,7 +594,7 @@ public abstract class AbstractJdbc1ResultSet
 		Field field;
 
 		if (columnIndex < 1 || columnIndex > fields.length)
-			throw new PSQLException("postgresql.res.colrange");
+			throw new PSQLException("postgresql.res.colrange", PSQLState.INVALID_PARAMETER_VALUE);
 		field = fields[columnIndex - 1];
 
 		// some fields can be null, mainly from those returned by MetaData methods
@@ -497,11 +635,18 @@ public abstract class AbstractJdbc1ResultSet
 				return getBytes(columnIndex);
 			default:
 				String type = field.getPGType();
+
 				// if the backend doesn't know the type then coerce to String
 				if (type.equals("unknown"))
 				{
 					return getString(columnIndex);
 				}
+                                // Specialized support for ref cursors is neater.
+                                else if (type.equals("refcursor"))
+                                {
+                                        String cursorName = getString(columnIndex);
+                                        return statement.createRefCursorResultSet(cursorName);
+                                }
 				else
 				{
 					return connection.getObject(field.getPGType(), getString(columnIndex));
@@ -548,21 +693,21 @@ public abstract class AbstractJdbc1ResultSet
 	 *
 	 * @return the next ResultSet, or null if there are none
 	 */
-	public java.sql.ResultSet getNext()
+	public ResultSet getNext()
 	{
-		return (java.sql.ResultSet)next;
+		return (ResultSet)next;
 	}
 
 	/*
 	 * This following method allows us to add a ResultSet object
 	 * to the end of the current chain.
 	 */
-	public void append(AbstractJdbc1ResultSet r)
+	public void append(BaseResultSet r)
 	{
 		if (next == null)
-			next = (java.sql.ResultSet)r;
+			next = r;
 		else
-			((AbstractJdbc1ResultSet)next).append(r);
+			next.append(r);
 	}
 
 	/*
@@ -653,7 +798,7 @@ public abstract class AbstractJdbc1ResultSet
 		// Handle Money
 		if (s.charAt(0) == '(')
 		{
-			s = "-" + org.postgresql.util.PGtokenizer.removePara(s).substring(1);
+			s = "-" + PGtokenizer.removePara(s).substring(1);
 		}
 		if (s.charAt(0) == '$')
 		{
@@ -672,7 +817,7 @@ public abstract class AbstractJdbc1ResultSet
 		if ( this_row == null )
 			throw new PSQLException("postgresql.res.nextrequired");
 		if ( column < 1 || column > fields.length )
-			throw new PSQLException("postgresql.res.colrange" );
+			throw new PSQLException("postgresql.res.colrange", PSQLState.INVALID_PARAMETER_VALUE );
 	}
 
 	//----------------- Formatting Methods -------------------
@@ -681,8 +826,19 @@ public abstract class AbstractJdbc1ResultSet
 	{
 		if (s != null)
 		{
-			int c = s.charAt(0);
-			return ((c == 't') || (c == 'T') || (c == '1'));
+			s = s.trim();
+
+			if (s.equalsIgnoreCase("true") || s.equalsIgnoreCase("t"))
+				return true;
+
+			try
+			{
+				if (Double.valueOf(s).doubleValue()==1)
+					return true;
+			}
+			catch (NumberFormatException e)
+			{
+			}
 		}
 		return false;		// SQL NULL
 	}
@@ -693,11 +849,12 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				s = s.trim();
 				return Integer.parseInt(s);
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException ("postgresql.res.badint", s);
+				throw new PSQLException ("postgresql.res.badint", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0;		// SQL NULL
@@ -709,11 +866,12 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				s = s.trim();		
 				return Long.parseLong(s);
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException ("postgresql.res.badlong", s);
+				throw new PSQLException ("postgresql.res.badlong", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0;		// SQL NULL
@@ -726,11 +884,12 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				s = s.trim();
 				val = new BigDecimal(s);
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException ("postgresql.res.badbigdec", s);
+				throw new PSQLException ("postgresql.res.badbigdec", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 			if (scale == -1)
 				return val;
@@ -740,7 +899,7 @@ public abstract class AbstractJdbc1ResultSet
 			}
 			catch (ArithmeticException e)
 			{
-				throw new PSQLException ("postgresql.res.badbigdec", s);
+				throw new PSQLException ("postgresql.res.badbigdec", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return null;		// SQL NULL
@@ -752,11 +911,12 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				s = s.trim();
 				return Float.valueOf(s).floatValue();
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException ("postgresql.res.badfloat", s);
+				throw new PSQLException ("postgresql.res.badfloat", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0;		// SQL NULL
@@ -768,11 +928,12 @@ public abstract class AbstractJdbc1ResultSet
 		{
 			try
 			{
+				s = s.trim();
 				return Double.valueOf(s).doubleValue();
 			}
 			catch (NumberFormatException e)
 			{
-				throw new PSQLException ("postgresql.res.baddouble", s);
+				throw new PSQLException ("postgresql.res.baddouble", PSQLState.NUMERIC_VALUE_OUT_OF_RANGE, s);
 			}
 		}
 		return 0;		// SQL NULL
@@ -786,20 +947,22 @@ public abstract class AbstractJdbc1ResultSet
 		// length >  10: SQL Timestamp, assumes PGDATESTYLE=ISO
 		try
 		{
+			s = s.trim();
 			return java.sql.Date.valueOf((s.length() == 10) ? s : s.substring(0, 10));
 		}
 		catch (NumberFormatException e)
 		{
-			throw new PSQLException("postgresql.res.baddate", s);
+			throw new PSQLException("postgresql.res.baddate",PSQLState.BAD_DATETIME_FORMAT, s);
 		}
 	}
 
-	public static Time toTime(String s, java.sql.ResultSet resultSet, String pgDataType) throws SQLException
+	public static Time toTime(String s, BaseResultSet resultSet, String pgDataType) throws SQLException
 	{
 		if (s == null)
 			return null; // SQL NULL
 		try
 		{
+			s = s.trim();
 			if (s.length() == 8)
 			{
 				//value is a time value
@@ -831,7 +994,7 @@ public abstract class AbstractJdbc1ResultSet
 		}
 		catch (NumberFormatException e)
 		{
-			throw new PSQLException("postgresql.res.badtime", s);
+			throw new PSQLException("postgresql.res.badtime", PSQLState.BAD_DATETIME_FORMAT, s);
 		}
 	}
 
@@ -860,32 +1023,34 @@ public abstract class AbstractJdbc1ResultSet
 	*
 	* @throws SQLException if there is a problem parsing s.
 	**/
-	public static Timestamp toTimestamp(String s, java.sql.ResultSet resultSet, String pgDataType)
+	public static Timestamp toTimestamp(String s, BaseResultSet resultSet, String pgDataType)
 	throws SQLException
 	{
-		AbstractJdbc1ResultSet rs = (AbstractJdbc1ResultSet)resultSet;
+		BaseResultSet rs = resultSet;
 		if (s == null)
 			return null;
 
+		s = s.trim();
 		// We must be synchronized here incase more theads access the ResultSet
 		// bad practice but possible. Anyhow this is to protect sbuf and
 		// SimpleDateFormat objects
 		synchronized (rs)
 		{
+			StringBuffer l_sbuf = rs.getStringBuffer();
 			SimpleDateFormat df = null;
-			if ( org.postgresql.Driver.logDebug )
-				org.postgresql.Driver.debug("the data from the DB is " + s);
+			if ( Driver.logDebug )
+				Driver.debug("the data from the DB is " + s);
 
 			// If first time, create the buffer, otherwise clear it.
-			if (rs.sbuf == null)
-				rs.sbuf = new StringBuffer(32);
+			if (l_sbuf == null)
+				l_sbuf = new StringBuffer(32);
 			else
 			{
-				rs.sbuf.setLength(0);
+				l_sbuf.setLength(0);
 			}
 
 			// Copy s into sbuf for parsing.
-			rs.sbuf.append(s);
+			l_sbuf.append(s);
 			int slen = s.length();
 
 			// For a Timestamp, the fractional seconds are stored in the
@@ -903,7 +1068,7 @@ public abstract class AbstractJdbc1ResultSet
 
 				// cut the copy to second value "2001-12-07 16:29:22"
 				int i = 19;
-				rs.sbuf.setLength(i);
+				l_sbuf.setLength(i);
 
 				char c = s.charAt(i++);
 				if (c == '.')
@@ -930,7 +1095,7 @@ public abstract class AbstractJdbc1ResultSet
 						}
 					catch (NumberFormatException e)
 						{
-							throw new PSQLException("postgresql.unusual", e);
+							throw new PSQLException("postgresql.unusual", PSQLState.UNEXPECTED_ERROR, e);
 						}
 
 					// The nanos field stores nanoseconds. Adjust the parsed
@@ -944,17 +1109,17 @@ public abstract class AbstractJdbc1ResultSet
 				{
 					// prepend the GMT part and then add the remaining bit of
 					// the string.
-					rs.sbuf.append(" GMT");
-					rs.sbuf.append(c);
-					rs.sbuf.append(s.substring(i, slen));
+					l_sbuf.append(" GMT");
+					l_sbuf.append(c);
+					l_sbuf.append(s.substring(i, slen));
 
 					// Lastly, if the tz part doesn't specify the :MM part then
 					// we add ":00" for java.
 					if (slen - i < 5)
-						rs.sbuf.append(":00");
+						l_sbuf.append(":00");
 
 					// we'll use this dateformat string to parse the result.
-					df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
+					df = rs.getTimestampTZFormat();
 				}
 				else
 				{
@@ -962,12 +1127,12 @@ public abstract class AbstractJdbc1ResultSet
 					//If timestamptz then we use GMT, else local timezone
 					if (pgDataType.equals("timestamptz"))
 					{
-						rs.sbuf.append(" GMT");
-						df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
+						l_sbuf.append(" GMT");
+						df = rs.getTimestampTZFormat();
 					}
 					else
 					{
-						df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
+						df = rs.getTimestampFormat();
 					}
 				}
 			}
@@ -977,12 +1142,12 @@ public abstract class AbstractJdbc1ResultSet
 				//If timestamptz then we use GMT, else local timezone
 				if (pgDataType.equals("timestamptz"))
 				{
-					rs.sbuf.append(" GMT");
-					df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
+					l_sbuf.append(" GMT");
+					df = rs.getTimestampTZFormat();
 				}
 				else
 				{
-					df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
+					df = rs.getTimestampFormat();
 				}
 			}
 			else
@@ -999,29 +1164,91 @@ public abstract class AbstractJdbc1ResultSet
 				// We must just have a date. This case is
 				// needed if this method is called on a date
 				// column
-				df = new SimpleDateFormat("yyyy-MM-dd");
+				df = rs.getDateFormat();
 			}
 
 			try
 			{
 				// All that's left is to parse the string and return the ts.
-				if ( org.postgresql.Driver.logDebug )
-					org.postgresql.Driver.debug("the data after parsing is " 
-                     + rs.sbuf.toString() + " with " + nanos + " nanos");
+				if ( Driver.logDebug )
+					Driver.debug("the data after parsing is " 
+                     + l_sbuf.toString() + " with " + nanos + " nanos");
 
-				Timestamp result = 
-					new Timestamp(df.parse(rs.sbuf.toString()).getTime());
+ 				Timestamp result = new Timestamp(df.parse(l_sbuf.toString()).getTime());
 				result.setNanos(nanos);
 				return result;
 			}
 			catch (ParseException e)
 			{
-				throw new PSQLException("postgresql.res.badtimestamp", new Integer(e.getErrorOffset()), s);
+				throw new PSQLException("postgresql.res.badtimestamp", PSQLState.BAD_DATETIME_FORMAT, new Integer(e.getErrorOffset()), s);
 			}
 		}
 	}
+	
+	private boolean isColumnTrimmable(int columnIndex) throws SQLException
+	{
+		switch (fields[columnIndex-1].getSQLType())
+		{
+			case Types.CHAR:
+			case Types.VARCHAR:
+		  	case Types.LONGVARCHAR:
+		   	case Types.BINARY:
+		   	case Types.VARBINARY:
+		   	case Types.LONGVARBINARY:
+		   		return true;
+		}
+	   	return false;
+	}
 
+	private byte[] trimBytes(int p_columnIndex, byte[] p_bytes) throws SQLException
+	{
+		int l_maxSize = statement.getMaxFieldSize();
+		//we need to trim if maxsize is set and the length is greater than maxsize and the
+		//type of this column is a candidate for trimming
+		if (l_maxSize > 0 && p_bytes.length > l_maxSize && isColumnTrimmable(p_columnIndex))
+		{
+			byte[] l_bytes = new byte[l_maxSize];
+			System.arraycopy (p_bytes, 0, l_bytes, 0, l_maxSize);
+			return l_bytes;
+		}
+		else
+		{
+			return p_bytes;
+		}
+	}
 
-
+	private String trimString(int p_columnIndex, String p_string) throws SQLException
+	{
+		int l_maxSize = statement.getMaxFieldSize();
+		//we need to trim if maxsize is set and the length is greater than maxsize and the
+		//type of this column is a candidate for trimming
+		if (l_maxSize > 0 && p_string.length() > l_maxSize && isColumnTrimmable(p_columnIndex))
+		{
+			return p_string.substring(0,l_maxSize);
+		} 
+		else 
+		{
+			return p_string;
+		}
+	}
+	
+	public SimpleDateFormat getTimestampTZFormat() {
+		if (m_tstzFormat == null) {
+			m_tstzFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
+		}
+		return m_tstzFormat;
+	}
+	public SimpleDateFormat getTimestampFormat() {
+		if (m_tsFormat == null) {
+			m_tsFormat = new SimpleDateFormat ("yyyy-MM-dd HH:mm:ss");
+		}
+		return m_tsFormat;
+	}
+	public SimpleDateFormat getDateFormat() {
+		if (m_dateFormat == null) {
+			m_dateFormat = new SimpleDateFormat("yyyy-MM-dd");
+		}
+		return m_dateFormat;
+	}
 }
 
